// Felix - non-threading example

#include <CLEye\CLEyeMulticam.h>
#include <opencv\cv.h>
#include <stdio.h>
#include <opencv\highgui.h>

#include <tchar.h>
#include <windows.h>
#pragma comment(lib, "CLEyeMulticam")
#pragma comment(lib, "opencv_core231d")
#pragma comment(lib, "opencv_highgui231d")


IplImage *pCapImage;
PBYTE pCapBuffer = NULL;
char windowName[64];

// Sample camera capture class
class CLEyeCameraCapture
{
	CHAR _windowName[256];
	GUID _cameraGUID;
	CLEyeCameraInstance _cam;
	CLEyeCameraColorMode _mode;
	CLEyeCameraResolution _resolution;
	float _fps;
	HANDLE _hThread;
	bool _running;
public:
	CLEyeCameraCapture(LPSTR windowName, GUID cameraGUID, CLEyeCameraColorMode mode, CLEyeCameraResolution resolution, float fps) :
		_cameraGUID(cameraGUID), _cam(NULL), _mode(mode), _resolution(resolution), _fps(fps), _running(false)
	{
		strcpy(_windowName, windowName);
	}

	void StopCapture()
	{
		if(!_running)	return;
		_running = false;
		WaitForSingleObject(_hThread, 1000);
		cvDestroyWindow(_windowName);
	}

	bool CreateCamera()
	{
		int w, h;
		
		// Create camera instance
		_cam = CLEyeCreateCamera(_cameraGUID, _mode, _resolution, _fps);
		if(_cam == NULL)		
			return 0;

		// Get camera frame dimensions
		CLEyeCameraGetFrameDimensions(_cam, w, h);	//CLEYE_QVGA - 320x240 image resolurion
													//CLEYE_VGA - 640x480 image resolution

		// Depending on color mode chosen, create the appropriate OpenCV image
		if(_mode == CLEYE_COLOR_PROCESSED || _mode == CLEYE_COLOR_RAW)
			pCapImage = cvCreateImage(cvSize(w, h), IPL_DEPTH_8U, 4); // Create depth if colour
		else
			pCapImage = cvCreateImage(cvSize(w, h), IPL_DEPTH_8U, 1); //grayscale

		// Set some camera parameters
		CLEyeSetCameraParameter(_cam, CLEYE_GAIN, 0);
		CLEyeSetCameraParameter(_cam, CLEYE_EXPOSURE, 511);//511);

		// Start capturing
		CLEyeCameraStart(_cam);
		cvGetImageRawData(pCapImage, &pCapBuffer);

		return 1;
	}

	IplImage* GetFrame()
	{
		// get frame
		CLEyeCameraGetFrame(_cam, pCapBuffer);
		return pCapImage;
	}

	void DestroyCamera()
	{
		// Stop camera capture
		CLEyeCameraStop(_cam);
		// Destroy camera object
		CLEyeDestroyCamera(_cam);

		_cam = NULL;
	}
};

CLEyeCameraCapture *cam[2] = {NULL}; //create object of camera class



int check_for_camera()
{
	// Query for number of connected cameras
	int numCams = CLEyeGetCameraCount();
	if(numCams == 0)
		return 0;

	printf("Found %d PS3Eye Camera(s)\n", numCams);

	for(int i = 0; i < numCams; i++)
	{
		// Query unique camera uuid
		GUID guid = CLEyeGetCameraUUID(i);
		printf("Camera %d GUID: [%08x-%04x-%04x-%02x%02x-%02x%02x%02x%02x%02x%02x]\n", 
						i+1, guid.Data1, guid.Data2, guid.Data3,
						guid.Data4[0], guid.Data4[1], guid.Data4[2],
						guid.Data4[3], guid.Data4[4], guid.Data4[5],
						guid.Data4[6], guid.Data4[7]);
		sprintf(windowName, "Camera Window %d", i+1);
		//Pass camera settings
		cam[i] = new CLEyeCameraCapture(windowName, guid, CLEYE_COLOR_PROCESSED, CLEYE_VGA, 60); 
	}

	return numCams;
}

void close_cameras(int numCams)
{
	// Close all cameras
	for(int i = 0; i < numCams; i++)
	{
		printf("Stopping capture on camera %d\n", i+1);
		cam[i]->StopCapture();
		delete cam[i];
	}
}

